@@ -42,23 +42,6 @@ Connecting
 
 See the API documentation on individual drivers for details on connecting.
     
-Data Types
-----------
-
-All of MySQL's standard types are supported.   These can also be specified within 
-table metadata, for the purpose of issuing CREATE TABLE statements
-which include MySQL-specific extensions.   The types are available
-from the module, as in::
-
-    from sqlalchemy.dialects import mysql
-
-    Table('mytable', metadata,
-          Column('id', Integer, primary_key=True),
-          Column('ittybittyblob', mysql.TINYBLOB),
-          Column('biggy', mysql.BIGINT(unsigned=True)))
-
-See the API documentation on specific column types for further details.
-
 Connection Timeouts
 -------------------
 
@@ -817,7 +800,7 @@ class NCHAR(_StringType, sqltypes.NCHAR):
     __visit_name__ = 'NCHAR'
 
     def __init__(self, length=None, **kwargs):
-        """Construct an NCHAR.  Arguments are:
+        """Construct an NCHAR.
 
         :param length: Maximum data length, in characters.
 
@@ -862,8 +845,6 @@ class ENUM(sqltypes.Enum, _StringType):
 
           Column('myenum', MSEnum("foo", "bar", "baz"))
 
-        Arguments are:
-
         :param enums: The range of valid values for this ENUM.  Values will be
           quoted when generating the schema according to the quoting flag (see
           below).
@@ -975,8 +956,6 @@ class SET(_StringType):
 
           Column('myset', MSSet("'foo'", "'bar'", "'baz'"))
 
-        Arguments are:
-
         :param values: The range of valid values for this SET.  Values will be
           used exactly as they appear when generating schemas.  Strings must
           be quoted, as in the example above.  Single-quotes are suggested for
@@ -1358,7 +1337,7 @@ class MySQLDDLCompiler(compiler.DDLCompiler):
         index = drop.element
         
         return "\nDROP INDEX %s ON %s" % \
-                    (self.preparer.quote(self._validate_identifier(index.name, False), index.quote),
+                    (self.preparer.quote(self._index_identifier(index.name), index.quote),
                      self.preparer.format_table(index.table))
 
     def visit_drop_constraint(self, drop):
@@ -1628,8 +1607,10 @@ class MySQLDialect(default.DefaultDialect):
     
     name = 'mysql'
     supports_alter = True
+    
     # identifiers are 64, however aliases can be 255...
     max_identifier_length = 255
+    max_index_name_length = 64
     
     supports_native_enum = True
     
@@ -1708,8 +1689,11 @@ class MySQLDialect(default.DefaultDialect):
 
     def is_disconnect(self, e):
         if isinstance(e, self.dbapi.OperationalError):
-            return self._extract_error_code(e) in (2006, 2013, 2014, 2045, 2055)
-        elif isinstance(e, self.dbapi.InterfaceError):  # if underlying connection is closed, this is the error you get
+            return self._extract_error_code(e) in \
+                        (2006, 2013, 2014, 2045, 2055)
+        elif isinstance(e, self.dbapi.InterfaceError):  
+            # if underlying connection is closed, 
+            # this is the error you get
             return "(0, '')" in str(e)
         else:
             return False
@@ -1760,7 +1744,7 @@ class MySQLDialect(default.DefaultDialect):
                 rs.close()
                 return have
             except exc.SQLError, e:
-                if self._extract_error_code(e) == 1146:
+                if self._extract_error_code(e.orig) == 1146:
                     return False
                 raise
         finally:
@@ -2048,7 +2032,7 @@ class MySQLDialect(default.DefaultDialect):
         try:
             rp = connection.execute(st)
         except exc.SQLError, e:
-            if self._extract_error_code(e) == 1146:
+            if self._extract_error_code(e.orig) == 1146:
                 raise exc.NoSuchTableError(full_name)
             else:
                 raise
@@ -2072,7 +2056,7 @@ class MySQLDialect(default.DefaultDialect):
             try:
                 rp = connection.execute(st)
             except exc.SQLError, e:
-                if self._extract_error_code(e) == 1146:
+                if self._extract_error_code(e.orig) == 1146:
                     raise exc.NoSuchTableError(full_name)
                 else:
                     raise
@@ -2387,8 +2371,8 @@ class MySQLTableDefinitionParser(object):
             r'(?: +COLLATE +(?P<collate>[\w_]+))?'
             r'(?: +(?P<notnull>NOT NULL))?'
             r'(?: +DEFAULT +(?P<default>'
-              r'(?:NULL|\x27(?:\x27\x27|[^\x27])*\x27|\w+)'
-              r'(?:ON UPDATE \w+)?'
+              r'(?:NULL|\x27(?:\x27\x27|[^\x27])*\x27|\w+'
+              r'(?: +ON UPDATE \w+)?)'
             r'))?'
             r'(?: +(?P<autoincr>AUTO_INCREMENT))?'
             r'(?: +COMMENT +(P<comment>(?:\x27\x27|[^\x27])+))?'
