@@ -35,18 +35,25 @@ NEVER_SET = util.symbol('NEVER_SET')
 
 # "passive" get settings
 # TODO: the True/False values need to be factored out
-# of the rest of ORM code
-# don't fire off any callables, and don't initialize the attribute to
-# an empty value
 PASSIVE_NO_INITIALIZE = True #util.symbol('PASSIVE_NO_INITIALIZE')
+"""Symbol indicating that loader callables should
+   not be fired off, and a non-initialized attribute 
+   should remain that way."""
 
-# don't fire off any callables, but if no callables present
-# then initialize to an empty value/collection
 # this is used by backrefs.
 PASSIVE_NO_FETCH = util.symbol('PASSIVE_NO_FETCH')
+"""Symbol indicating that loader callables should not be fired off.
+   Non-initialized attributes should be initialized to an empty value."""
+
+PASSIVE_ONLY_PERSISTENT = util.symbol('PASSIVE_ONLY_PERSISTENT')
+"""Symbol indicating that loader callables should only fire off for
+persistent objects.
+
+Loads of "previous" values during change events use this flag.
+"""
 
-# fire callables/initialize as needed
 PASSIVE_OFF = False #util.symbol('PASSIVE_OFF')
+"""Symbol indicating that loader callables should be executed."""
 
 INSTRUMENTATION_MANAGER = '__sa_instrumentation_manager__'
 """Attribute, elects custom instrumentation when present on a mapped class.
@@ -96,7 +103,8 @@ class QueryableAttribute(interfaces.PropComparator):
         """Construct an InstrumentedAttribute.
 
           comparator
-            a sql.Comparator to which class-level compare/math events will be sent
+            a sql.Comparator to which class-level compare/math events will be
+            sent
         """
         self.key = key
         self.impl = impl
@@ -104,7 +112,8 @@ class QueryableAttribute(interfaces.PropComparator):
         self.parententity = parententity
 
     def get_history(self, instance, **kwargs):
-        return self.impl.get_history(instance_state(instance), instance_dict(instance), **kwargs)
+        return self.impl.get_history(instance_state(instance),
+                                        instance_dict(instance), **kwargs)
 
     def __selectable__(self):
         # TODO: conditionally attach this method based on clause_element ?
@@ -148,7 +157,8 @@ class InstrumentedAttribute(QueryableAttribute):
     """Public-facing descriptor, placed in the mapped class dictionary."""
 
     def __set__(self, instance, value):
-        self.impl.set(instance_state(instance), instance_dict(instance), value, None)
+        self.impl.set(instance_state(instance), 
+                        instance_dict(instance), value, None)
 
     def __delete__(self, instance):
         self.impl.delete(instance_state(instance), instance_dict(instance))
@@ -156,7 +166,8 @@ class InstrumentedAttribute(QueryableAttribute):
     def __get__(self, instance, owner):
         if instance is None:
             return self
-        return self.impl.get(instance_state(instance), instance_dict(instance))
+        return self.impl.get(instance_state(instance),
+                                instance_dict(instance))
 
 class _ProxyImpl(object):
     accepts_scalar_loader = False
@@ -205,7 +216,8 @@ def proxied_attribute_factory(descriptor):
             return descriptor.__delete__(instance)
 
         def __getattr__(self, attribute):
-            """Delegate __getattr__ to the original descriptor and/or comparator."""
+            """Delegate __getattr__ to the original descriptor and/or
+            comparator."""
             
             try:
                 return getattr(descriptor, attribute)
@@ -214,10 +226,10 @@ def proxied_attribute_factory(descriptor):
                     return getattr(self._comparator, attribute)
                 except AttributeError:
                     raise AttributeError(
-                            'Neither %r object nor %r object has an attribute %r' % (
-                            type(descriptor).__name__, 
-                            type(self._comparator).__name__, 
-                            attribute)
+                    'Neither %r object nor %r object has an attribute %r' % (
+                    type(descriptor).__name__, 
+                    type(self._comparator).__name__, 
+                    attribute)
                     )
 
     Proxy.__name__ = type(descriptor).__name__ + 'Proxy'
@@ -294,6 +306,7 @@ class AttributeImpl(object):
         self.active_history = active_history
         self.expire_missing = expire_missing
         
+        
     def hasparent(self, state, optimistic=False):
         """Return the boolean value of a `hasparent` flag attached to 
         the given state.
@@ -386,7 +399,7 @@ class AttributeImpl(object):
 
             # Return a new, empty value
             return self.initialize(state, dict_)
-
+    
     def append(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         self.set(state, dict_, value, initiator, passive=passive)
 
@@ -422,6 +435,7 @@ class ScalarAttributeImpl(AttributeImpl):
 
     accepts_scalar_loader = True
     uses_objects = False
+    supports_population = True
 
     def delete(self, state, dict_):
 
@@ -441,7 +455,7 @@ class ScalarAttributeImpl(AttributeImpl):
             self, state, dict_.get(self.key, NO_VALUE))
 
     def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         if self.active_history:
@@ -450,7 +464,8 @@ class ScalarAttributeImpl(AttributeImpl):
             old = dict_.get(self.key, NO_VALUE)
 
         if self.extensions:
-            value = self.fire_replace_event(state, dict_, value, old, initiator)
+            value = self.fire_replace_event(state, dict_, 
+                                                value, old, initiator)
         state.modified_event(dict_, self, False, old)
         dict_[self.key] = value
 
@@ -469,11 +484,13 @@ class ScalarAttributeImpl(AttributeImpl):
 
 
 class MutableScalarAttributeImpl(ScalarAttributeImpl):
-    """represents a scalar value-holding InstrumentedAttribute, which can detect
-    changes within the value itself.
+    """represents a scalar value-holding InstrumentedAttribute, which can
+    detect changes within the value itself.
+
     """
 
     uses_objects = False
+    supports_population = True
 
     def __init__(self, class_, key, callable_,
                     class_manager, copy_function=None,
@@ -517,12 +534,13 @@ class MutableScalarAttributeImpl(ScalarAttributeImpl):
         state.mutable_dict.pop(self.key)
 
     def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         if self.extensions:
             old = self.get(state, dict_)
-            value = self.fire_replace_event(state, dict_, value, old, initiator)
+            value = self.fire_replace_event(state, dict_, 
+                                            value, old, initiator)
 
         state.modified_event(dict_, self, True, NEVER_SET)
         dict_[self.key] = value
@@ -539,18 +557,19 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
 
     accepts_scalar_loader = False
     uses_objects = True
+    supports_population = True
 
     def __init__(self, class_, key, callable_, 
                     trackparent=False, extension=None, copy_function=None,
                     compare_function=None, **kwargs):
         super(ScalarObjectAttributeImpl, self).__init__(
-                                                class_, 
-                                                key,
-                                                callable_, 
-                                                trackparent=trackparent, 
-                                                extension=extension,
-                                                compare_function=compare_function, 
-                                                **kwargs)
+                                            class_, 
+                                            key,
+                                            callable_, 
+                                            trackparent=trackparent, 
+                                            extension=extension,
+                                            compare_function=compare_function, 
+                                            **kwargs)
         if compare_function is None:
             self.is_equal = identity_equal
 
@@ -577,14 +596,14 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
         setter operation.
 
         """
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         if self.active_history:
-            old = self.get(state, dict_)
+            old = self.get(state, dict_, passive=PASSIVE_ONLY_PERSISTENT)
         else:
             old = self.get(state, dict_, passive=PASSIVE_NO_FETCH)
-             
+        
         value = self.fire_replace_event(state, dict_, value, old, initiator)
         dict_[self.key] = value
 
@@ -603,7 +622,7 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
                 previous is not None and
                 previous is not PASSIVE_NO_RESULT):
                 self.sethasparent(instance_state(previous), False)
-
+        
         for ext in self.extensions:
             value = ext.set(state, value, previous, initiator or self)
 
@@ -623,24 +642,25 @@ class CollectionAttributeImpl(AttributeImpl):
 
     InstrumentedCollectionAttribute holds an arbitrary, user-specified
     container object (defaulting to a list) and brokers access to the
-    CollectionAdapter, a "view" onto that object that presents consistent
-    bag semantics to the orm layer independent of the user data implementation.
+    CollectionAdapter, a "view" onto that object that presents consistent bag
+    semantics to the orm layer independent of the user data implementation.
 
     """
     accepts_scalar_loader = False
     uses_objects = True
+    supports_population = True
 
     def __init__(self, class_, key, callable_, 
                     typecallable=None, trackparent=False, extension=None,
                     copy_function=None, compare_function=None, **kwargs):
         super(CollectionAttributeImpl, self).__init__(
-                                                class_, 
-                                                key, 
-                                                callable_, 
-                                                trackparent=trackparent,
-                                                extension=extension,
-                                                compare_function=compare_function, 
-                                                **kwargs)
+                                            class_, 
+                                            key, 
+                                            callable_, 
+                                            trackparent=trackparent,
+                                            extension=extension,
+                                            compare_function=compare_function, 
+                                            **kwargs)
 
         if copy_function is None:
             copy_function = self.__copy
@@ -661,7 +681,8 @@ class CollectionAttributeImpl(AttributeImpl):
         for ext in self.extensions:
             value = ext.append(state, value, initiator or self)
 
-        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
+        state.modified_event(dict_, self, True, 
+                                NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
 
         if self.trackparent and value is not None:
             self.sethasparent(instance_state(value), True)
@@ -669,7 +690,8 @@ class CollectionAttributeImpl(AttributeImpl):
         return value
 
     def fire_pre_remove_event(self, state, dict_, initiator):
-        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
+        state.modified_event(dict_, self, True, 
+                                NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
 
     def fire_remove_event(self, state, dict_, value, initiator):
         if self.trackparent and value is not None:
@@ -678,7 +700,8 @@ class CollectionAttributeImpl(AttributeImpl):
         for ext in self.extensions:
             ext.remove(state, value, initiator or self)
 
-        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
+        state.modified_event(dict_, self, True, 
+                                NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
 
     def delete(self, state, dict_):
         if self.key not in dict_:
@@ -703,25 +726,27 @@ class CollectionAttributeImpl(AttributeImpl):
             self.key, state, self.collection_factory)
 
     def append(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         collection = self.get_collection(state, dict_, passive=passive)
         if collection is PASSIVE_NO_RESULT:
             value = self.fire_append_event(state, dict_, value, initiator)
-            assert self.key not in dict_, "Collection was loaded during event handling."
+            assert self.key not in dict_, \
+                    "Collection was loaded during event handling."
             state.get_pending(self.key).append(value)
         else:
             collection.append_with_event(value, initiator)
 
     def remove(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         collection = self.get_collection(state, state.dict, passive=passive)
         if collection is PASSIVE_NO_RESULT:
             self.fire_remove_event(state, dict_, value, initiator)
-            assert self.key not in dict_, "Collection was loaded during event handling."
+            assert self.key not in dict_, \
+                    "Collection was loaded during event handling."
             state.get_pending(self.key).remove(value)
         else:
             collection.remove_with_event(value, initiator)
@@ -734,7 +759,7 @@ class CollectionAttributeImpl(AttributeImpl):
         setter operation.
         """
 
-        if initiator is self:
+        if initiator and initiator.parent_token is self.parent_token:
             return
 
         self._set_iterable(
@@ -759,15 +784,16 @@ class CollectionAttributeImpl(AttributeImpl):
         else:
             new_values = list(iterable)
 
-        old = self.get(state, dict_)
-
-        # ignore re-assignment of the current collection, as happens
-        # implicitly with in-place operators (foo.collection |= other)
-        if old is iterable:
+        old = self.get(state, dict_, passive=PASSIVE_ONLY_PERSISTENT)
+        if old is PASSIVE_NO_RESULT:
+            old = self.initialize(state, dict_)
+        elif old is iterable:
+            # ignore re-assignment of the current collection, as happens
+            # implicitly with in-place operators (foo.collection |= other)
             return
 
         state.modified_event(dict_, self, True, old)
-
+        
         old_collection = self.get_collection(state, dict_, old)
 
         dict_[self.key] = user_data
@@ -806,7 +832,8 @@ class CollectionAttributeImpl(AttributeImpl):
 
         return user_data
 
-    def get_collection(self, state, dict_, user_data=None, passive=PASSIVE_OFF):
+    def get_collection(self, state, dict_, 
+                            user_data=None, passive=PASSIVE_OFF):
         """Retrieve the CollectionAdapter associated with the given state.
 
         Creates a new CollectionAdapter if one does not exist.
@@ -836,11 +863,12 @@ class GenericBackrefExtension(interfaces.AttributeExtension):
     def set(self, state, child, oldchild, initiator):
         if oldchild is child:
             return child
-            
+
         if oldchild is not None and oldchild is not PASSIVE_NO_RESULT:
             # With lazy=None, there's no guarantee that the full collection is
             # present when updating via a backref.
-            old_state, old_dict = instance_state(oldchild), instance_dict(oldchild)
+            old_state, old_dict = instance_state(oldchild),\
+                                    instance_dict(oldchild)
             impl = old_state.get_impl(self.key)
             try:
                 impl.remove(old_state, 
@@ -851,31 +879,37 @@ class GenericBackrefExtension(interfaces.AttributeExtension):
                 pass
                 
         if child is not None:
-            child_state, child_dict = instance_state(child), instance_dict(child)
+            child_state, child_dict = instance_state(child),\
+                                        instance_dict(child)
             child_state.get_impl(self.key).append(
                                             child_state, 
                                             child_dict, 
                                             state.obj(), 
-                                            initiator, passive=PASSIVE_NO_FETCH)
+                                            initiator, 
+                                            passive=PASSIVE_NO_FETCH)
         return child
 
     def append(self, state, child, initiator):
-        child_state, child_dict = instance_state(child), instance_dict(child)
+        child_state, child_dict = instance_state(child), \
+                                    instance_dict(child)
         child_state.get_impl(self.key).append(
                                             child_state, 
                                             child_dict, 
                                             state.obj(), 
-                                            initiator, passive=PASSIVE_NO_FETCH)
+                                            initiator, 
+                                            passive=PASSIVE_NO_FETCH)
         return child
 
     def remove(self, state, child, initiator):
         if child is not None:
-            child_state, child_dict = instance_state(child), instance_dict(child)
+            child_state, child_dict = instance_state(child),\
+                                        instance_dict(child)
             child_state.get_impl(self.key).remove(
                                             child_state, 
                                             child_dict, 
                                             state.obj(), 
-                                            initiator, passive=PASSIVE_NO_FETCH)
+                                            initiator,
+                                            passive=PASSIVE_NO_FETCH)
 
 
 class Events(object):
@@ -918,12 +952,11 @@ class ClassManager(dict):
         self.class_ = class_
         self.factory = None  # where we came from, for inheritance bookkeeping
         self.info = {}
-        self.mapper = None
         self.new_init = None
         self.mutable_attributes = set()
         self.local_attrs = {}
         self.originals = {}
-        for base in class_.__mro__[-2:0:-1]:   # reverse, skipping 1st and last
+        for base in class_.__mro__[-2:0:-1]:  # reverse, skipping 1st and last
             if not isinstance(base, type):
                 continue
             cls_state = manager_of_class(base)
@@ -933,6 +966,22 @@ class ClassManager(dict):
         self.manage()
         self._instrument_init()
     
+    @property
+    def is_mapped(self):
+        return 'mapper' in self.__dict__
+        
+    @util.memoized_property
+    def mapper(self):
+        raise exc.UnmappedClassError(self.class_)
+        
+    def _attr_has_impl(self, key):
+        """Return True if the given attribute is fully initialized.
+        
+        i.e. has an impl.
+        """
+        
+        return key in self and self[key].impl is not None
+        
     def _configure_create_arguments(self, 
                             _source=None, 
                             deferred_scalar_loader=None):
@@ -955,7 +1004,8 @@ class ClassManager(dict):
             self.deferred_scalar_loader = deferred_scalar_loader
     
     def _subclass_manager(self, cls):
-        """Create a new ClassManager for a subclass of this ClassManager's class.
+        """Create a new ClassManager for a subclass of this ClassManager's
+        class.
         
         This is called automatically when attributes are instrumented so that
         the attributes can be propagated to subclasses against their own
@@ -1050,7 +1100,8 @@ class ClassManager(dict):
     def install_descriptor(self, key, inst):
         if key in (self.STATE_ATTR, self.MANAGER_ATTR):
             raise KeyError("%r: requested attribute name conflicts with "
-                           "instrumentation attribute of the same name." % key)
+                           "instrumentation attribute of the same name." %
+                           key)
         setattr(self.class_, key, inst)
 
     def uninstall_descriptor(self, key):
@@ -1059,7 +1110,8 @@ class ClassManager(dict):
     def install_member(self, key, implementation):
         if key in (self.STATE_ATTR, self.MANAGER_ATTR):
             raise KeyError("%r: requested attribute name conflicts with "
-                           "instrumentation attribute of the same name." % key)
+                           "instrumentation attribute of the same name." %
+                           key)
         self.originals.setdefault(key, getattr(self.class_, key, None))
         setattr(self.class_, key, implementation)
 
@@ -1094,11 +1146,13 @@ class ClassManager(dict):
 
     def new_instance(self, state=None):
         instance = self.class_.__new__(self.class_)
-        setattr(instance, self.STATE_ATTR, state or self._create_instance_state(instance))
+        setattr(instance, self.STATE_ATTR, 
+                    state or self._create_instance_state(instance))
         return instance
 
     def setup_instance(self, instance, state=None):
-        setattr(instance, self.STATE_ATTR, state or self._create_instance_state(instance))
+        setattr(instance, self.STATE_ATTR, 
+                    state or self._create_instance_state(instance))
     
     def teardown_instance(self, instance):
         delattr(instance, self.STATE_ATTR)
@@ -1111,6 +1165,15 @@ class ClassManager(dict):
         """
         if hasattr(instance, self.STATE_ATTR):
             return False
+        elif self.class_ is not instance.__class__ and \
+                self.is_mapped:
+            # this will create a new ClassManager for the
+            # subclass, without a mapper.  This is likely a
+            # user error situation but allow the object
+            # to be constructed, so that it is usable
+            # in a non-ORM context at least.
+            return self._subclass_manager(instance.__class__).\
+                        _new_state_if_none(instance)
         else:
             state = self._create_instance_state(instance)
             setattr(instance, self.STATE_ATTR, state)
@@ -1192,7 +1255,8 @@ class _ClassInstrumentationAdapter(ClassManager):
         if delegate:
             return delegate(key, state, factory)
         else:
-            return ClassManager.initialize_collection(self, key, state, factory)
+            return ClassManager.initialize_collection(self, key, 
+                                                        state, factory)
 
     def new_instance(self, state=None):
         instance = self.class_.__new__(self.class_)
@@ -1237,8 +1301,10 @@ class _ClassInstrumentationAdapter(ClassManager):
         return self._get_dict
 
 class History(tuple):
-    """A 3-tuple of added, unchanged and deleted values.
-
+    """A 3-tuple of added, unchanged and deleted values,
+    representing the changes which have occured on an instrumented
+    attribute.
+    
     Each tuple member is an iterable sequence.
 
     """
@@ -1246,9 +1312,18 @@ class History(tuple):
     __slots__ = ()
 
     added = property(itemgetter(0))
+    """Return the collection of items added to the attribute (the first tuple
+    element)."""
+    
     unchanged = property(itemgetter(1))
+    """Return the collection of items that have not changed on the attribute
+    (the second tuple element)."""
+    
+    
     deleted = property(itemgetter(2))
-
+    """Return the collection of items that have been removed from the
+    attribute (the third tuple element)."""
+    
     def __new__(cls, added, unchanged, deleted):
         return tuple.__new__(cls, (added, unchanged, deleted))
     
@@ -1256,25 +1331,38 @@ class History(tuple):
         return self != HISTORY_BLANK
     
     def empty(self):
+        """Return True if this :class:`History` has no changes
+        and no existing, unchanged state.
+        
+        """
+        
         return not bool(
                         (self.added or self.deleted)
                         or self.unchanged and self.unchanged != [None]
                     ) 
         
     def sum(self):
+        """Return a collection of added + unchanged + deleted."""
+        
         return (self.added or []) +\
                 (self.unchanged or []) +\
                 (self.deleted or [])
     
     def non_deleted(self):
+        """Return a collection of added + unchanged."""
+        
         return (self.added or []) +\
                 (self.unchanged or [])
     
     def non_added(self):
+        """Return a collection of unchanged + deleted."""
+        
         return (self.unchanged or []) +\
                 (self.deleted or [])
     
     def has_changes(self):
+        """Return True if this :class:`History` has changes."""
+        
         return bool(self.added or self.deleted)
         
     def as_state(self):
@@ -1335,11 +1423,19 @@ class History(tuple):
 HISTORY_BLANK = History(None, None, None)
 
 def get_history(obj, key, **kwargs):
-    """Return a History record for the given object and attribute key.
+    """Return a :class:`.History` record for the given object 
+    and attribute key.
     
-    obj is an instrumented object instance.  An InstanceState
-    is accepted directly for backwards compatibility but 
-    this usage is deprecated.
+    :param obj: an object whose class is instrumented by the
+      attributes package.  
+    
+    :param key: string attribute name.
+    
+    :param kwargs: Optional keyword arguments currently
+      include the ``passive`` flag, which indicates if the attribute should be
+      loaded from the database if not already present (:attr:`PASSIVE_NO_FETCH`), and
+      if the attribute should be not initialized to a blank value otherwise
+      (:attr:`PASSIVE_NO_INITIALIZE`). Default is :attr:`PASSIVE_OFF`.
     
     """
     return get_state_history(instance_state(obj), key, **kwargs)
@@ -1375,7 +1471,8 @@ def register_attribute(class_, key, **kw):
     comparator = kw.pop('comparator', None)
     parententity = kw.pop('parententity', None)
     doc = kw.pop('doc', None)
-    register_descriptor(class_, key, proxy_property, comparator, parententity, doc=doc)
+    register_descriptor(class_, key, proxy_property, 
+                            comparator, parententity, doc=doc)
     if not proxy_property:
         register_attribute_impl(class_, key, **kw)
     
@@ -1408,6 +1505,7 @@ def register_attribute_impl(class_, key,
     manager[key].impl = impl
     
     manager.post_configure_attribute(key)
+
     
 def register_descriptor(class_, key, proxy_property=None, comparator=None, 
                                 parententity=None, property_=None, doc=None):
@@ -1417,7 +1515,8 @@ def register_descriptor(class_, key, proxy_property=None, comparator=None,
         proxy_type = proxied_attribute_factory(proxy_property)
         descriptor = proxy_type(key, proxy_property, comparator, parententity)
     else:
-        descriptor = InstrumentedAttribute(key, comparator=comparator, parententity=parententity)
+        descriptor = InstrumentedAttribute(key, comparator=comparator,
+                                            parententity=parententity)
     
     descriptor.__doc__ = doc
         
@@ -1436,7 +1535,8 @@ def init_collection(obj, key):
         for elem in values:
             collection_adapter.append_without_event(elem)
     
-    For an easier way to do the above, see :func:`~sqlalchemy.orm.attributes.set_committed_value`.
+    For an easier way to do the above, see
+     :func:`~sqlalchemy.orm.attributes.set_committed_value`.
     
     obj is an instrumented object instance.  An InstanceState
     is accepted directly for backwards compatibility but 
@@ -1512,14 +1612,15 @@ def del_attribute(instance, key):
     state.get_impl(key).delete(state, dict_)
 
 def is_instrumented(instance, key):
-    """Return True if the given attribute on the given instance is instrumented
-    by the attributes package.
+    """Return True if the given attribute on the given instance is
+    instrumented by the attributes package.
     
     This function may be used regardless of instrumentation
     applied directly to the class, i.e. no descriptors are required.
     
     """
-    return manager_of_class(instance.__class__).is_instrumented(key, search=True)
+    return manager_of_class(instance.__class__).\
+                        is_instrumented(key, search=True)
 
 class InstrumentationRegistry(object):
     """Private instrumentation registration singleton.
@@ -1574,11 +1675,12 @@ class InstrumentationRegistry(object):
         return manager
 
     def _collect_management_factories_for(self, cls):
-        """Return a collection of factories in play or specified for a hierarchy.
+        """Return a collection of factories in play or specified for a
+        hierarchy.
 
-        Traverses the entire inheritance graph of a cls and returns a collection
-        of instrumentation factories for those classes.  Factories are extracted
-        from active ClassManagers, if available, otherwise
+        Traverses the entire inheritance graph of a cls and returns a
+        collection of instrumentation factories for those classes. Factories
+        are extracted from active ClassManagers, if available, otherwise
         instrumentation_finders is consulted.
 
         """
@@ -1600,7 +1702,8 @@ class InstrumentationRegistry(object):
         return factories
 
     def manager_of_class(self, cls):
-        # this is only called when alternate instrumentation has been established
+        # this is only called when alternate instrumentation 
+        # has been established
         if cls is None:
             return None
         try:
@@ -1611,22 +1714,26 @@ class InstrumentationRegistry(object):
             return finder(cls)
 
     def state_of(self, instance):
-        # this is only called when alternate instrumentation has been established
+        # this is only called when alternate instrumentation 
+        # has been established
         if instance is None:
             raise AttributeError("None has no persistent state.")
         try:
             return self._state_finders[instance.__class__](instance)
         except KeyError:
-            raise AttributeError("%r is not instrumented" % instance.__class__)
+            raise AttributeError("%r is not instrumented" %
+                                    instance.__class__)
 
     def dict_of(self, instance):
-        # this is only called when alternate instrumentation has been established
+        # this is only called when alternate instrumentation 
+        # has been established
         if instance is None:
             raise AttributeError("None has no persistent state.")
         try:
             return self._dict_finders[instance.__class__](instance)
         except KeyError:
-            raise AttributeError("%r is not instrumented" % instance.__class__)
+            raise AttributeError("%r is not instrumented" %
+                                    instance.__class__)
         
     def unregister(self, class_):
         if class_ in self._manager_finders:
