@@ -1,18 +1,23 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from sqlalchemy.test.testing import assert_raises
+from sqlalchemy.test.testing import assert_raises_message
+from sqlalchemy.test.testing import emits_warning
+
 import pickle
-from sqlalchemy import Integer, String, UniqueConstraint, CheckConstraint,\
-                        ForeignKey, MetaData, Sequence, ForeignKeyConstraint,\
-                        ColumnDefault
+from sqlalchemy import Integer, String, UniqueConstraint, \
+    CheckConstraint, ForeignKey, MetaData, Sequence, \
+    ForeignKeyConstraint, ColumnDefault, Index
 from sqlalchemy.test.schema import Table, Column
 from sqlalchemy import schema, exc
 import sqlalchemy as tsa
-from sqlalchemy.test import TestBase, ComparesTables, AssertsCompiledSQL, testing, engines
+from sqlalchemy.test import TestBase, ComparesTables, \
+    AssertsCompiledSQL, testing, engines
 from sqlalchemy.test.testing import eq_
 
 class MetaDataTest(TestBase, ComparesTables):
     def test_metadata_connect(self):
         metadata = MetaData()
-        t1 = Table('table1', metadata, Column('col1', Integer, primary_key=True),
+        t1 = Table('table1', metadata, 
+            Column('col1', Integer, primary_key=True),
             Column('col2', String(20)))
         metadata.bind = testing.db
         metadata.create_all()
@@ -42,12 +47,15 @@ class MetaDataTest(TestBase, ComparesTables):
             Column('foo', String(), nullable=False),
             Column('baz', String(), unique=True),
             Column(Integer(), primary_key=True),
-            Column('bar', Integer(), Sequence('foo_seq'), primary_key=True, key='bar'),
+            Column('bar', Integer(), Sequence('foo_seq'), primary_key=True,
+                                                            key='bar'),
             Column(Integer(), ForeignKey('bat.blah')),
-            Column('bar', Integer(), ForeignKey('bat.blah'), primary_key=True, key='bar'),
+            Column('bar', Integer(), ForeignKey('bat.blah'), primary_key=True,
+                                                            key='bar'),
         ]:
             c2 = col.copy()
-            for attr in ('name', 'type', 'nullable', 'primary_key', 'key', 'unique'):
+            for attr in ('name', 'type', 'nullable', 
+                        'primary_key', 'key', 'unique'):
                 eq_(getattr(col, attr), getattr(c2, attr))
             eq_(len(col.foreign_keys), len(c2.foreign_keys))
             if col.default:
@@ -71,7 +79,8 @@ class MetaDataTest(TestBase, ComparesTables):
         
     def test_dupe_tables(self):
         metadata = MetaData()
-        t1 = Table('table1', metadata, Column('col1', Integer, primary_key=True),
+        t1 = Table('table1', metadata, 
+            Column('col1', Integer, primary_key=True),
             Column('col2', String(20)))
 
         metadata.bind = testing.db
@@ -79,11 +88,16 @@ class MetaDataTest(TestBase, ComparesTables):
         try:
             try:
                 t1 = Table('table1', metadata, autoload=True)
-                t2 = Table('table1', metadata, Column('col1', Integer, primary_key=True),
+                t2 = Table('table1', metadata, 
+                    Column('col1', Integer, primary_key=True),
                     Column('col2', String(20)))
                 assert False
             except tsa.exc.InvalidRequestError, e:
-                assert str(e) == "Table 'table1' is already defined for this MetaData instance.  Specify 'useexisting=True' to redefine options and columns on an existing Table object."
+                assert str(e) \
+                    == "Table 'table1' is already defined for this "\
+                    "MetaData instance.  Specify 'useexisting=True' "\
+                    "to redefine options and columns on an existing "\
+                    "Table object."
         finally:
             metadata.drop_all()
     
@@ -123,9 +137,12 @@ class MetaDataTest(TestBase, ComparesTables):
         table = Table('mytable', meta,
             Column('myid', Integer, Sequence('foo_id_seq'), primary_key=True),
             Column('name', String(40), nullable=True),
-            Column('foo', String(40), nullable=False, server_default='x', server_onupdate='q'),
-            Column('bar', String(40), nullable=False, default='y', onupdate='z'),
-            Column('description', String(30), CheckConstraint("description='hi'")),
+            Column('foo', String(40), nullable=False, server_default='x',
+                                                        server_onupdate='q'),
+            Column('bar', String(40), nullable=False, default='y',
+                                                        onupdate='z'),
+            Column('description', String(30),
+                                    CheckConstraint("description='hi'")),
             UniqueConstraint('name'),
             test_needs_fk=True,
         )
@@ -164,33 +181,37 @@ class MetaDataTest(TestBase, ComparesTables):
 
         meta.create_all(testing.db)
         try:
-            for test, has_constraints, reflect in ((test_to_metadata, True, False), (test_pickle, True, False),(test_pickle_via_reflect, False, True)):
+            for test, has_constraints, reflect in (test_to_metadata,
+                    True, False), (test_pickle, True, False), \
+                (test_pickle_via_reflect, False, True):
                 table_c, table2_c = test()
                 self.assert_tables_equal(table, table_c)
                 self.assert_tables_equal(table2, table2_c)
-
                 assert table is not table_c
                 assert table.primary_key is not table_c.primary_key
-                assert list(table2_c.c.myid.foreign_keys)[0].column is table_c.c.myid
-                assert list(table2_c.c.myid.foreign_keys)[0].column is not table.c.myid
+                assert list(table2_c.c.myid.foreign_keys)[0].column \
+                    is table_c.c.myid
+                assert list(table2_c.c.myid.foreign_keys)[0].column \
+                    is not table.c.myid
                 assert 'x' in str(table_c.c.foo.server_default.arg)
-                
                 if not reflect:
                     assert isinstance(table_c.c.myid.default, Sequence)
                     assert str(table_c.c.foo.server_onupdate.arg) == 'q'
                     assert str(table_c.c.bar.default.arg) == 'y'
-                    assert getattr(table_c.c.bar.onupdate.arg, 'arg', table_c.c.bar.onupdate.arg) == 'z'
+                    assert getattr(table_c.c.bar.onupdate.arg, 'arg',
+                                   table_c.c.bar.onupdate.arg) == 'z'
                     assert isinstance(table2_c.c.id.default, Sequence)
-                    
-                # constraints dont get reflected for any dialect right now
+
+                # constraints dont get reflected for any dialect right
+                # now
+
                 if has_constraints:
                     for c in table_c.c.description.constraints:
                         if isinstance(c, CheckConstraint):
                             break
                     else:
                         assert False
-                    assert str(c.sqltext)=="description='hi'"
-
+                    assert str(c.sqltext) == "description='hi'"
                     for c in table_c.constraints:
                         if isinstance(c, UniqueConstraint):
                             break
@@ -207,7 +228,8 @@ class MetaDataTest(TestBase, ComparesTables):
         table = Table('mytable', meta,
             Column('myid', Integer, primary_key=True),
             Column('name', String(40), nullable=True),
-            Column('description', String(30), CheckConstraint("description='hi'")),
+            Column('description', String(30),
+                            CheckConstraint("description='hi'")),
             UniqueConstraint('name'),
             test_needs_fk=True,
         )
@@ -222,8 +244,66 @@ class MetaDataTest(TestBase, ComparesTables):
         table_c = table.tometadata(meta2, schema='someschema')
         table2_c = table2.tometadata(meta2, schema='someschema')
 
-        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid == table2_c.c.myid))
-        eq_(str(table_c.join(table2_c).onclause), "someschema.mytable.myid = someschema.othertable.myid")
+        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid
+            == table2_c.c.myid))
+        eq_(str(table_c.join(table2_c).onclause),
+            'someschema.mytable.myid = someschema.othertable.myid')
+
+    def test_tometadata_kwargs(self):
+        meta = MetaData()
+
+        table = Table('mytable', meta,
+            Column('myid', Integer, primary_key=True),
+            mysql_engine='InnoDB',
+        )
+
+        meta2 = MetaData()
+        table_c = table.tometadata(meta2)
+
+        eq_(table.kwargs,table_c.kwargs)
+
+    def test_tometadata_indexes(self):
+        meta = MetaData()
+
+        table = Table('mytable', meta,
+            Column('id', Integer, primary_key=True),
+            Column('data1', Integer, index=True),
+            Column('data2', Integer),
+        )
+        Index('multi',table.c.data1,table.c.data2),
+        
+        meta2 = MetaData()
+        table_c = table.tometadata(meta2)
+
+        def _get_key(i):
+            return [i.name,i.unique] + \
+                    sorted(i.kwargs.items()) + \
+                    i.columns.keys()
+        
+        eq_(
+            sorted([_get_key(i) for i in table.indexes]),
+            sorted([_get_key(i) for i in table_c.indexes])
+        )
+
+    @emits_warning("Table '.+' already exists within the given MetaData")
+    def test_tometadata_already_there(self):
+        
+        meta1 = MetaData()
+        table1 = Table('mytable', meta1,
+            Column('myid', Integer, primary_key=True),
+        )
+        meta2 = MetaData()
+        table2 = Table('mytable', meta2,
+            Column('yourid', Integer, primary_key=True),
+        )
+
+        meta3 = MetaData()
+        
+        table_c = table1.tometadata(meta2)
+        table_d = table2.tometadata(meta2)
+
+        # d'oh!
+        assert table_c is table_d
 
     def test_tometadata_default_schema(self):
         meta = MetaData()
@@ -231,7 +311,8 @@ class MetaDataTest(TestBase, ComparesTables):
         table = Table('mytable', meta,
             Column('myid', Integer, primary_key=True),
             Column('name', String(40), nullable=True),
-            Column('description', String(30), CheckConstraint("description='hi'")),
+            Column('description', String(30),
+                        CheckConstraint("description='hi'")),
             UniqueConstraint('name'),
             test_needs_fk=True,
             schema='myschema',
@@ -248,8 +329,10 @@ class MetaDataTest(TestBase, ComparesTables):
         table_c = table.tometadata(meta2)
         table2_c = table2.tometadata(meta2)
 
-        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid == table2_c.c.myid))
-        eq_(str(table_c.join(table2_c).onclause), "myschema.mytable.myid = myschema.othertable.myid")
+        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid
+            == table2_c.c.myid))
+        eq_(str(table_c.join(table2_c).onclause),
+            'myschema.mytable.myid = myschema.othertable.myid')
 
     def test_manual_dependencies(self):
         meta = MetaData()
@@ -276,7 +359,8 @@ class MetaDataTest(TestBase, ComparesTables):
         table = Table('mytable', meta,
             Column('myid', Integer, primary_key=True),
             Column('name', String(40), nullable=True),
-            Column('description', String(30), CheckConstraint("description='hi'")),
+            Column('description', String(30),
+                        CheckConstraint("description='hi'")),
             UniqueConstraint('name'),
             test_needs_fk=True,
         )
@@ -291,8 +375,10 @@ class MetaDataTest(TestBase, ComparesTables):
         table_c = table.tometadata(meta2, schema=None)
         table2_c = table2.tometadata(meta2, schema=None)
 
-        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid == table2_c.c.myid))
-        eq_(str(table_c.join(table2_c).onclause), "mytable.myid = othertable.myid")
+        eq_(str(table_c.join(table2_c).onclause), str(table_c.c.myid
+            == table2_c.c.myid))
+        eq_(str(table_c.join(table2_c).onclause),
+            'mytable.myid = othertable.myid')
 
     def test_nonexistent(self):
         assert_raises(tsa.exc.NoSuchTableError, Table,
