@@ -1,13 +1,12 @@
-from sqlalchemy.test.testing import eq_
+from sqlalchemy.test.testing import eq_, assert_raises
 import time
 import weakref
 from sqlalchemy import select, MetaData, Integer, String, pool
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from sqlalchemy.test.schema import Table, Column
 import sqlalchemy as tsa
 from sqlalchemy.test import TestBase, testing, engines
 from sqlalchemy.test.util import gc_collect
-
+from sqlalchemy import exc
 
 class MockDisconnect(Exception):
     pass
@@ -62,28 +61,33 @@ class MockReconnectTest(TestBase):
         db.dialect.is_disconnect = lambda e: isinstance(e, MockDisconnect)
 
     def test_reconnect(self):
-        """test that an 'is_disconnect' condition will invalidate the connection, and additionally
-        dispose the previous connection pool and recreate."""
-
+        """test that an 'is_disconnect' condition will invalidate the
+        connection, and additionally dispose the previous connection
+        pool and recreate."""
 
         pid = id(db.pool)
 
         # make a connection
+
         conn = db.connect()
 
         # connection works
+
         conn.execute(select([1]))
 
-        # create a second connection within the pool, which we'll ensure also goes away
+        # create a second connection within the pool, which we'll ensure
+        # also goes away
+
         conn2 = db.connect()
         conn2.close()
 
         # two connections opened total now
+
         assert len(dbapi.connections) == 2
 
         # set it to fail
-        dbapi.shutdown()
 
+        dbapi.shutdown()
         try:
             conn.execute(select([1]))
             assert False
@@ -91,19 +95,20 @@ class MockReconnectTest(TestBase):
             pass
 
         # assert was invalidated
+
         assert not conn.closed
         assert conn.invalidated
 
         # close shouldnt break
-        conn.close()
 
+        conn.close()
         assert id(db.pool) != pid
 
         # ensure all connections closed (pool was recycled)
+
         gc_collect()
         assert len(dbapi.connections) == 0
-
-        conn =db.connect()
+        conn = db.connect()
         conn.execute(select([1]))
         conn.close()
         assert len(dbapi.connections) == 1
@@ -112,7 +117,6 @@ class MockReconnectTest(TestBase):
         conn = db.connect()
         trans = conn.begin()
         dbapi.shutdown()
-
         try:
             conn.execute(select([1]))
             assert False
@@ -120,34 +124,32 @@ class MockReconnectTest(TestBase):
             pass
 
         # assert was invalidated
+
         gc_collect()
         assert len(dbapi.connections) == 0
         assert not conn.closed
         assert conn.invalidated
         assert trans.is_active
-
         try:
             conn.execute(select([1]))
             assert False
         except tsa.exc.InvalidRequestError, e:
-            assert str(e) == "Can't reconnect until invalid transaction is rolled back"
-
+            assert str(e) \
+                == "Can't reconnect until invalid transaction is "\
+                "rolled back"
         assert trans.is_active
-
         try:
             trans.commit()
             assert False
         except tsa.exc.InvalidRequestError, e:
-            assert str(e) == "Can't reconnect until invalid transaction is rolled back"
-
+            assert str(e) \
+                == "Can't reconnect until invalid transaction is "\
+                "rolled back"
         assert trans.is_active
-
         trans.rollback()
         assert not trans.is_active
-
         conn.execute(select([1]))
         assert not conn.invalidated
-
         assert len(dbapi.connections) == 1
 
     def test_conn_reusable(self):
@@ -219,6 +221,8 @@ class RealReconnectTest(TestBase):
     def teardown(self):
         engine.dispose()
 
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_reconnect(self):
         conn = engine.connect()
 
@@ -255,8 +259,43 @@ class RealReconnectTest(TestBase):
 
         conn.close()
 
+    def test_invalidate_twice(self):
+        conn = engine.connect()
+        conn.invalidate()
+        conn.invalidate()
+    
+    def test_explode_in_initializer(self):
+        engine = engines.testing_engine()
+        def broken_initialize(connection):
+            connection.execute("select fake_stuff from _fake_table")
+            
+        engine.dialect.initialize = broken_initialize
+        
+        # raises a DBAPIError, not an AttributeError
+        assert_raises(exc.DBAPIError, engine.connect)
+
+        # dispose connections so we get a new one on
+        # next go
+        engine.dispose()
+
+        p1 = engine.pool
+        
+        def is_disconnect(e):
+            return True
+            
+        engine.dialect.is_disconnect = is_disconnect
+
+        # invalidate() also doesn't screw up
+        assert_raises(exc.DBAPIError, engine.connect)
+        
+        # pool was recreated
+        assert engine.pool is not p1
+
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_null_pool(self):
-        engine = engines.reconnecting_engine(options=dict(poolclass=pool.NullPool))
+        engine = \
+            engines.reconnecting_engine(options=dict(poolclass=pool.NullPool))
         conn = engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
@@ -272,6 +311,8 @@ class RealReconnectTest(TestBase):
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.invalidated
         
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_close(self):
         conn = engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
@@ -290,62 +331,56 @@ class RealReconnectTest(TestBase):
         conn = engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
 
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_with_transaction(self):
         conn = engine.connect()
-
         trans = conn.begin()
-
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.closed
-
         engine.test_shutdown()
-
         try:
             conn.execute(select([1]))
             assert False
         except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
-
         assert not conn.closed
         assert conn.invalidated
         assert trans.is_active
-
         try:
             conn.execute(select([1]))
             assert False
         except tsa.exc.InvalidRequestError, e:
-            assert str(e) == "Can't reconnect until invalid transaction is rolled back"
-
+            assert str(e) \
+                == "Can't reconnect until invalid transaction is "\
+                "rolled back"
         assert trans.is_active
-
         try:
             trans.commit()
             assert False
         except tsa.exc.InvalidRequestError, e:
-            assert str(e) == "Can't reconnect until invalid transaction is rolled back"
-
+            assert str(e) \
+                == "Can't reconnect until invalid transaction is "\
+                "rolled back"
         assert trans.is_active
-
         trans.rollback()
         assert not trans.is_active
-
         assert conn.invalidated
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.invalidated
 
 class RecycleTest(TestBase):
+
     def test_basic(self):
-        for threadlocal in (False, True):
-            engine = engines.reconnecting_engine(options={'pool_recycle':1, 'pool_threadlocal':threadlocal})
-        
+        for threadlocal in False, True:
+            engine = engines.reconnecting_engine(options={'pool_recycle'
+                    : 1, 'pool_threadlocal': threadlocal})
             conn = engine.contextual_connect()
             eq_(conn.execute(select([1])).scalar(), 1)
             conn.close()
-
             engine.test_shutdown()
             time.sleep(2)
-    
             conn = engine.contextual_connect()
             eq_(conn.execute(select([1])).scalar(), 1)
             conn.close()
@@ -368,22 +403,24 @@ class InvalidateDuringResultTest(TestBase):
         meta.drop_all()
         engine.dispose()
 
-    @testing.fails_on('+mysqldb', "Buffers the result set and doesn't check for connection close")
-    @testing.fails_on('+pg8000', "Buffers the result set and doesn't check for connection close")
+    @testing.fails_on('+mysqldb',
+                      "Buffers the result set and doesn't check for "
+                      "connection close")
+    @testing.fails_on('+pg8000',
+                      "Buffers the result set and doesn't check for "
+                      "connection close")
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_invalidate_on_results(self):
         conn = engine.connect()
-
-        result = conn.execute("select * from sometable")
+        result = conn.execute('select * from sometable')
         for x in xrange(20):
             result.fetchone()
-
         engine.test_shutdown()
         try:
-            print "ghost result: %r" % result.fetchone()
+            print 'ghost result: %r' % result.fetchone()
             assert False
         except tsa.exc.DBAPIError, e:
             if not e.connection_invalidated:
                 raise
-
         assert conn.invalidated
-
