@@ -17,11 +17,13 @@ from sqlalchemy.test.util import round_decimal
 
 class AdaptTest(TestBase):
     def test_uppercase_rendering(self):
-        """Test that uppercase types from types.py always render as their type.
+        """Test that uppercase types from types.py always render as their
+        type.
         
-        As of SQLA 0.6, using an uppercase type means you want specifically that
-        type.  If the database in use doesn't support that DDL, it (the DB backend) 
-        should raise an error - it means you should be using a lowercased (genericized) type.
+        As of SQLA 0.6, using an uppercase type means you want specifically
+        that type. If the database in use doesn't support that DDL, it (the DB
+        backend) should raise an error - it means you should be using a
+        lowercased (genericized) type.
         
         """
         
@@ -30,20 +32,21 @@ class AdaptTest(TestBase):
                 mysql.dialect(), 
                 postgresql.dialect(), 
                 sqlite.dialect(), 
-                mssql.dialect()]: # TODO when dialects are complete:  engines.all_dialects():
+                mssql.dialect()]: 
             for type_, expected in (
                 (FLOAT, "FLOAT"),
                 (NUMERIC, "NUMERIC"),
                 (DECIMAL, "DECIMAL"),
                 (INTEGER, "INTEGER"),
                 (SMALLINT, "SMALLINT"),
-                (TIMESTAMP, "TIMESTAMP"),
+                (TIMESTAMP, ("TIMESTAMP", "TIMESTAMP WITHOUT TIME ZONE")),
                 (DATETIME, "DATETIME"),
                 (DATE, "DATE"),
-                (TIME, "TIME"),
+                (TIME, ("TIME", "TIME WITHOUT TIME ZONE")),
                 (CLOB, "CLOB"),
                 (VARCHAR(10), ("VARCHAR(10)","VARCHAR(10 CHAR)")),
-                (NVARCHAR(10), ("NVARCHAR(10)", "NATIONAL VARCHAR(10)", "NVARCHAR2(10)")),
+                (NVARCHAR(10), ("NVARCHAR(10)", "NATIONAL VARCHAR(10)",
+                                    "NVARCHAR2(10)")),
                 (CHAR, "CHAR"),
                 (NCHAR, ("NCHAR", "NATIONAL CHAR")),
                 (BLOB, "BLOB"),
@@ -51,14 +54,18 @@ class AdaptTest(TestBase):
             ):
                 if isinstance(expected, str):
                     expected = (expected, )
-                for exp in expected:
-                    compiled = types.to_instance(type_).compile(dialect=dialect)
-                    if exp in compiled:
-                        break
-                else:
-                    assert False, "%r matches none of %r for dialect %s" % \
-                                            (compiled, expected, dialect.name)
-            
+
+                compiled = types.to_instance(type_).\
+                            compile(dialect=dialect)
+                    
+                assert compiled in expected, \
+                    "%r matches none of %r for dialect %s" % \
+                    (compiled, expected, dialect.name)
+                
+                assert str(types.to_instance(type_)) in expected, \
+                    "default str() of type %r not expected, %r" % \
+                    (type_, expected)
+                
 class TypeAffinityTest(TestBase):
     def test_type_affinity(self):
         for type_, affin in [
@@ -111,7 +118,7 @@ class PickleMetadataTest(TestBase):
                 mt = loads(dumps(meta))
                 
 
-class UserDefinedTest(TestBase):
+class UserDefinedTest(TestBase, AssertsCompiledSQL):
     """tests user-defined types."""
 
     def test_processing(self):
@@ -141,6 +148,116 @@ class UserDefinedTest(TestBase):
             for col in row[3], row[4]:
                 assert isinstance(col, unicode)
 
+    def test_typedecorator_impl(self):
+        for impl_, exp, kw in [
+            (Float, "FLOAT", {}),
+            (Float, "FLOAT(2)", {'precision':2}),
+            (Float(2), "FLOAT(2)", {'precision':4}),
+            (Numeric(19, 2), "NUMERIC(19, 2)", {}),
+        ]:
+            for dialect_ in (postgresql, mssql, mysql):
+                dialect_ = dialect_.dialect()
+                
+                raw_impl = types.to_instance(impl_, **kw)
+                
+                class MyType(types.TypeDecorator):
+                    impl = impl_
+                
+                dec_type = MyType(**kw)
+                
+                eq_(dec_type.impl.__class__, raw_impl.__class__)
+                
+                raw_dialect_impl = raw_impl.dialect_impl(dialect_)
+                dec_dialect_impl = dec_type.dialect_impl(dialect_)
+                eq_(dec_dialect_impl.__class__, MyType)
+                eq_(raw_dialect_impl.__class__ , dec_dialect_impl.impl.__class__)
+                
+                self.assert_compile(
+                    MyType(**kw),
+                    exp,
+                    dialect=dialect_
+                )
+    
+    def test_user_defined_typedec_impl(self):
+        class MyType(types.TypeDecorator):
+            impl = Float
+            
+            def load_dialect_impl(self, dialect):
+                if dialect.name == 'sqlite':
+                    return String(50)
+                else:
+                    return super(MyType, self).load_dialect_impl(dialect)
+        
+        sl = sqlite.dialect()
+        pg = postgresql.dialect()
+        t = MyType()
+        self.assert_compile(t, "VARCHAR(50)", dialect=sl)
+        self.assert_compile(t, "FLOAT", dialect=pg)
+        eq_(
+            t.dialect_impl(dialect=sl).impl.__class__, 
+            String().dialect_impl(dialect=sl).__class__
+        )
+        eq_(
+                t.dialect_impl(dialect=pg).impl.__class__, 
+                Float().dialect_impl(pg).__class__
+        )
+    
+    @testing.provide_metadata
+    def test_type_coerce(self):
+        """test ad-hoc usage of custom types with type_coerce()."""
+        
+        class MyType(types.TypeDecorator):
+            impl = String
+
+            def process_bind_param(self, value, dialect):
+                return value[0:-8]
+            
+            def process_result_value(self, value, dialect):
+                return value + "BIND_OUT"
+        
+        t = Table('t', metadata, Column('data', String(50)))
+        metadata.create_all()
+        
+        t.insert().values(data=type_coerce('d1BIND_OUT',MyType)).execute()
+
+        eq_(
+            select([type_coerce(t.c.data, MyType)]).execute().fetchall(),
+            [('d1BIND_OUT', )]
+        )
+        
+        eq_(
+            select([t.c.data, type_coerce(t.c.data, MyType)]).execute().fetchall(),
+            [('d1', 'd1BIND_OUT')]
+        )
+        
+        eq_(
+            select([t.c.data, type_coerce(t.c.data, MyType)]).\
+                        where(type_coerce(t.c.data, MyType) == 'd1BIND_OUT').\
+                        execute().fetchall(),
+            [('d1', 'd1BIND_OUT')]
+        )
+
+        eq_(
+            select([t.c.data, type_coerce(t.c.data, MyType)]).\
+                        where(t.c.data == type_coerce('d1BIND_OUT', MyType)).\
+                        execute().fetchall(),
+            [('d1', 'd1BIND_OUT')]
+        )
+
+        eq_(
+            select([t.c.data, type_coerce(t.c.data, MyType)]).\
+                        where(t.c.data == type_coerce(None, MyType)).\
+                        execute().fetchall(),
+            []
+        )
+
+        eq_(
+            select([t.c.data, type_coerce(t.c.data, MyType)]).\
+                        where(type_coerce(t.c.data, MyType) == None).\
+                        execute().fetchall(),
+            []
+        )
+        
     @classmethod
     def setup_class(cls):
         global users, metadata
@@ -562,7 +679,17 @@ class EnumTest(TestBase):
                 (3, 'one'),
             ]
         )
-
+    
+    def test_adapt(self):
+        from sqlalchemy.dialects.postgresql import ENUM
+        e1 = Enum('one','two','three', native_enum=False)
+        eq_(e1.adapt(ENUM).native_enum, False)
+        e1 = Enum('one','two','three', native_enum=True)
+        eq_(e1.adapt(ENUM).native_enum, True)
+        e1 = Enum('one','two','three', name='foo', schema='bar')
+        eq_(e1.adapt(ENUM).name, 'foo')
+        eq_(e1.adapt(ENUM).schema, 'bar')
+        
     @testing.fails_on('mysql+mysqldb', "MySQL seems to issue a 'data truncated' warning.")
     def test_constraint(self):
         assert_raises(exc.DBAPIError, 
@@ -721,10 +848,10 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         meta.create_all()
 
         test_table.insert().execute({
-                                        'id':1, 
-                                        'data':'somedata', 
-                                        'atimestamp':datetime.date(2007, 10, 15), 
-                                        'avalue':25, 'bvalue':'foo'})
+                                'id':1, 
+                                'data':'somedata', 
+                                'atimestamp':datetime.date(2007, 10, 15), 
+                                'avalue':25, 'bvalue':'foo'})
 
     @classmethod
     def teardown_class(cls):
@@ -735,7 +862,8 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         eq_(
             test_table.select().execute().fetchall(),
-            [(1, 'somedata', datetime.date(2007, 10, 15), 25, "BIND_INfooBIND_OUT")]
+            [(1, 'somedata', datetime.date(2007, 10, 15), 25,
+             'BIND_INfooBIND_OUT')]
         )
 
     def test_bind_adapt(self):
@@ -745,9 +873,9 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         eq_(
             testing.db.execute(
-                            select([test_table.c.id, test_table.c.data, test_table.c.atimestamp])
-                            .where(expr), 
-                            {"thedate":datetime.date(2007, 10, 15)}).fetchall(),
+                    select([test_table.c.id, test_table.c.data, test_table.c.atimestamp])
+                    .where(expr), 
+                    {"thedate":datetime.date(2007, 10, 15)}).fetchall(),
             [(1, 'somedata', datetime.date(2007, 10, 15))]
         )
 
@@ -755,21 +883,25 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         eq_(expr.right.type._type_affinity, MyCustomType)
 
         eq_(
-            testing.db.execute(test_table.select().where(expr), {"somevalue":25}).fetchall(),
-            [(1, 'somedata', datetime.date(2007, 10, 15), 25, 'BIND_INfooBIND_OUT')]
+            testing.db.execute(test_table.select().where(expr),
+             {'somevalue': 25}).fetchall(),
+            [(1, 'somedata', datetime.date(2007, 10, 15), 25,
+             'BIND_INfooBIND_OUT')]
         )
 
         expr = test_table.c.bvalue == bindparam("somevalue")
         eq_(expr.right.type._type_affinity, String)
         
         eq_(
-            testing.db.execute(test_table.select().where(expr), {"somevalue":"foo"}).fetchall(),
-            [(1, 'somedata', datetime.date(2007, 10, 15), 25, 'BIND_INfooBIND_OUT')]
+            testing.db.execute(test_table.select().where(expr), 
+                {"somevalue":"foo"}).fetchall(),
+            [(1, 'somedata', 
+                datetime.date(2007, 10, 15), 25, 'BIND_INfooBIND_OUT')]
         )
     
     def test_literal_adapt(self):
-        # literals get typed based on the types dictionary, unless compatible
-        # with the left side type
+        # literals get typed based on the types dictionary, unless
+        # compatible with the left side type
 
         expr = column('foo', String) == 5
         eq_(expr.right.type._type_affinity, Integer)
@@ -816,8 +948,9 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     def test_typedec_operator_adapt(self):
         expr = test_table.c.bvalue + "hi"
         
-        assert expr.type.__class__ is String
-
+        assert expr.type.__class__ is MyTypeDec
+        assert expr.right.type.__class__ is MyTypeDec
+        
         eq_(
             testing.db.execute(select([expr.label('foo')])).scalar(),
             "BIND_INfooBIND_INhiBIND_OUT"
@@ -842,7 +975,7 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             use_default_dialect=True
         )
         
-        assert expr.type.__class__ is String
+        assert expr.type.__class__ is MyTypeDec
         eq_(
             testing.db.execute(select([expr.label('foo')])).scalar(),
             "BIND_INfooBIND_IN6BIND_OUT"
@@ -894,6 +1027,34 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         expr = func.current_date() - column('foo', types.TIMESTAMP)
         eq_(expr.type._type_affinity, types.Interval)
     
+    def test_numerics_coercion(self):
+        from sqlalchemy.sql import column
+        import operator
+        
+        for op in (
+            operator.add,
+            operator.mul,
+            operator.truediv,
+            operator.sub
+        ):
+            for other in (Numeric(10, 2), Integer):
+                expr = op(
+                        column('bar', types.Numeric(10, 2)),
+                        column('foo', other)
+                       )
+                assert isinstance(expr.type, types.Numeric)
+                expr = op(
+                        column('foo', other),
+                        column('bar', types.Numeric(10, 2))
+                       )
+                assert isinstance(expr.type, types.Numeric)
+
+    def test_null_comparison(self):
+        eq_(
+            str(column('a', types.NullType()) + column('b', types.NullType())), 
+            "a + b"
+        )
+        
     def test_expression_typing(self):
         expr = column('bar', Integer) - 3
         
@@ -912,6 +1073,23 @@ class ExpressionTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         assert distinct(test_table.c.data).type == test_table.c.data.type
         assert test_table.c.data.distinct().type == test_table.c.data.type
     
+class CompileTest(TestBase, AssertsCompiledSQL):
+    def test_default_compile(self):
+        """test that the base dialect of the type object is used
+        for default compilation.
+        
+        """
+        for type_, expected in (
+            (String(), "VARCHAR"),
+            (Integer(), "INTEGER"),
+            (postgresql.INET(), "INET"),
+            (postgresql.FLOAT(), "FLOAT"),
+            (mysql.REAL(precision=8, scale=2), "REAL(8, 2)"),
+            (postgresql.REAL(), "REAL"),
+            (INTEGER(), "INTEGER"),
+            (mysql.INTEGER(display_width=5), "INTEGER(5)")
+        ):
+            self.assert_compile(type_, expected)
 
 class DateTest(TestBase, AssertsExecutionResults):
     @classmethod
@@ -1066,7 +1244,7 @@ class NumericTest(TestBase):
     def teardown(self):
         metadata.drop_all()
         
-    @testing.emits_warning(r".*does \*not\* support Decimal objects natively")    
+    @testing.emits_warning(r".*does \*not\* support Decimal objects natively")
     def _do_test(self, type_, input_, output, filter_ = None):
         t = Table('t', metadata, Column('x', type_))
         t.create()
